package pps.core.space.service;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import pps.core.space.entity.SpaceCalibrationHistoryEnt;
import pps.core.space.entity.SpaceCalibrationHistoryView;
import pps.core.space.entity.SpaceCalibrationPeriodEnt;
import pps.core.space.entity.SpaceCalibrationPeriodView;
import pps.core.space.mapper.SpaceCalibrationHistoryMapper;
import pps.core.space.mapper.SpaceCalibrationHistoryViewMapper;
import pps.core.space.mapper.SpaceCalibrationPeriodMapper;
import pps.core.space.mapper.SpaceCalibrationPeriodViewMapper;
import pps.core.space.service.data.space_calibration_history.*;
import pps.core.space.service.data.space_calibration_period.CreateSpaceCalibrationPeriodInput;
import pps.core.space.service.data.space_calibration_period.GetSpaceCalibrationPeriodViewInput;
import pps.core.space.service.data.space_calibration_period.GetSpaceCalibrationPeriodViewOutput;
import pps.core.space.service.data.space_calibration_period.UpdateSpaceCalibrationPeriodInput;
import xstartup.annotation.XService;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.base.data.CustomQueryInput;
import xstartup.base.util.XCopyUtils;
import xstartup.data.XPageResult;
import xstartup.data.XServiceResult;
import xstartup.data.XSingleResult;
import xstartup.error.XError;
import xstartup.feature.api.annotation.XApiAnonymous;
import xstartup.feature.api.annotation.XApiGet;
import xstartup.feature.api.annotation.XApiPost;
import xstartup.feature.mybatis.helper.XMapperHelper;
import xstartup.helper.XTransactionHelper;

import java.util.List;

/**
 * 定期校准模块
 *
 * @author ZWT
 * @date 2023/09/06
 */
@XService
@XText("定期校准模块")
public class SpaceCalibrationService {

    /**
     * 校准周期--新增
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XServiceResult}
     */
    @XApiAnonymous
    @XApiPost
    @XText("校准周期--新增")
    public XServiceResult createSpaceCalibrationPeriod(XContext context, CreateSpaceCalibrationPeriodInput input) {
        return XTransactionHelper.begin(context, () -> {
            SpaceCalibrationPeriodMapper mapper = context.getBean(SpaceCalibrationPeriodMapper.class);
            SpaceCalibrationPeriodEnt entity = new SpaceCalibrationPeriodEnt();
            XCopyUtils.copyObject(input, entity);
            mapper.insert(entity);
            return XServiceResult.OK;
        });
    }

    /**
     * 校准周期--修改
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XServiceResult}
     */
    @XApiAnonymous
    @XApiPost
    @XText("校准周期--修改")
    public XServiceResult updateSpaceCalibrationPeriod(XContext context, UpdateSpaceCalibrationPeriodInput input) {
        SpaceCalibrationPeriodMapper mapper = context.getBean(SpaceCalibrationPeriodMapper.class);
        QueryWrapper<SpaceCalibrationPeriodEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(SpaceCalibrationPeriodEnt::getId, input.getId());
        SpaceCalibrationPeriodEnt entity = mapper.selectOne(queryWrapper);
        if (entity == null) {
            return XServiceResult.error(context, XError.NotFound);
        }
        XCopyUtils.copyObject(input, entity);
        mapper.updateById(entity);
        return XServiceResult.OK;
    }

    /**
     * 校准周期--详情
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XSingleResult}<{@link GetSpaceCalibrationPeriodViewOutput}>
     */
    @XApiAnonymous
    @XApiGet
    @XText("校准周期--详情")
    public XSingleResult<GetSpaceCalibrationPeriodViewOutput> getSpaceCalibrationPeriodView(XContext context, GetSpaceCalibrationPeriodViewInput input) {
        SpaceCalibrationPeriodViewMapper mapper = context.getBean(SpaceCalibrationPeriodViewMapper.class);
        SpaceCalibrationPeriodView record = new SpaceCalibrationPeriodView();
        XCopyUtils.copyObject(input, record);
        SpaceCalibrationPeriodView view = mapper.selectOne(record);
        if (view == null) {
            return XSingleResult.error(context, XError.NotFound);
        }
        GetSpaceCalibrationPeriodViewOutput output = new GetSpaceCalibrationPeriodViewOutput();
        XCopyUtils.copyObject(view, output);
        return XSingleResult.success(output);
    }

    @XApiAnonymous
    @XApiPost
    public XServiceResult createSpaceCalibrationHistory(XContext context, CreateSpaceCalibrationHistoryInput input) {
        SpaceCalibrationHistoryMapper mapper = context.getBean(SpaceCalibrationHistoryMapper.class);
        SpaceCalibrationHistoryEnt entity = new SpaceCalibrationHistoryEnt();
        XCopyUtils.copyObject(input, entity);
        mapper.insert(entity);
        return XServiceResult.OK;
    }

    @XApiAnonymous
    @XApiPost
    public XServiceResult updateSpaceCalibrationHistory(XContext context, UpdateSpaceCalibrationHistoryInput input) {
        SpaceCalibrationHistoryMapper mapper = context.getBean(SpaceCalibrationHistoryMapper.class);
        QueryWrapper<SpaceCalibrationHistoryEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(SpaceCalibrationHistoryEnt::getId, input.getId());
        SpaceCalibrationHistoryEnt entity = mapper.selectOne(queryWrapper);
        if (entity == null) {
            return XServiceResult.error(context, XError.NotFound);
        }
        XCopyUtils.copyObject(input, entity);
        mapper.updateById(entity);
        return XServiceResult.OK;
    }

    @XApiAnonymous
    @XApiPost
    public XServiceResult deleteSpaceCalibrationHistory(XContext context, DeleteSpaceCalibrationHistoryInput input) {
        SpaceCalibrationHistoryMapper mapper = context.getBean(SpaceCalibrationHistoryMapper.class);
        QueryWrapper<SpaceCalibrationHistoryEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(SpaceCalibrationHistoryEnt::getId, input.getId());
        SpaceCalibrationHistoryEnt entity = mapper.selectOne(queryWrapper);
        if (entity == null) {
            return XServiceResult.error(context, XError.NotFound);
        }
        mapper.deleteById(entity);
        return XServiceResult.OK;
    }

    @XApiAnonymous
    @XApiGet
    public XSingleResult<GetSpaceCalibrationHistoryOutput> getSpaceCalibrationHistory(XContext context, GetSpaceCalibrationHistoryInput input) {
        SpaceCalibrationHistoryMapper mapper = context.getBean(SpaceCalibrationHistoryMapper.class);
        QueryWrapper<SpaceCalibrationHistoryEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(SpaceCalibrationHistoryEnt::getId, input.getId());
        SpaceCalibrationHistoryEnt entity = mapper.selectOne(queryWrapper);
        if (entity == null) {
            return XSingleResult.error(context, XError.NotFound);
        }
        GetSpaceCalibrationHistoryOutput output = new GetSpaceCalibrationHistoryOutput();
        XCopyUtils.copyObject(entity, output);
        return XSingleResult.success(output);
    }

    @XApiAnonymous
    @XApiGet
    public XPageResult<QuerySpaceCalibrationHistoryOutput> querySpaceCalibrationHistory(XContext context, QuerySpaceCalibrationHistoryInput input) {
        SpaceCalibrationHistoryMapper mapper = context.getBean(SpaceCalibrationHistoryMapper.class);
        QueryWrapper<SpaceCalibrationHistoryEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().like(SpaceCalibrationHistoryEnt::getId, input.getId())
                .eq(SpaceCalibrationHistoryEnt::getIsDeleted, input.getIsDeleted())
                .like(SpaceCalibrationHistoryEnt::getCreateById, input.getCreateById())
                .like(SpaceCalibrationHistoryEnt::getCreateByName, input.getCreateByName())
                .eq(SpaceCalibrationHistoryEnt::getCreateTime, input.getCreateTime())
                .like(SpaceCalibrationHistoryEnt::getModifyById, input.getModifyById())
                .like(SpaceCalibrationHistoryEnt::getModifyByName, input.getModifyByName())
                .eq(SpaceCalibrationHistoryEnt::getModifyTime, input.getModifyTime())
                .like(SpaceCalibrationHistoryEnt::getLineId, input.getLineId())
                .like(SpaceCalibrationHistoryEnt::getInstitutionId, input.getInstitutionId())
                .like(SpaceCalibrationHistoryEnt::getExecutionCycle, input.getExecutionCycle())
                .eq(SpaceCalibrationHistoryEnt::getCalibrationDate, input.getCalibrationDate());
        return XMapperHelper.query(mapper, input, queryWrapper, QuerySpaceCalibrationHistoryOutput.class);
    }

    @XApiAnonymous
    @XApiGet
    public XPageResult<QuerySpaceCalibrationHistoryOutput> dynamicQuerySpaceCalibrationHistory(XContext context, CustomQueryInput input) {
        SpaceCalibrationHistoryMapper mapper = context.getBean(SpaceCalibrationHistoryMapper.class);
        return XMapperHelper.query(mapper, input, SpaceCalibrationHistoryEnt.class, QuerySpaceCalibrationHistoryOutput.class);
    }

    @XApiAnonymous
    @XApiGet
    public XSingleResult<GetSpaceCalibrationHistoryViewOutput> getSpaceCalibrationHistoryView(XContext context, GetSpaceCalibrationHistoryViewInput input) {
        SpaceCalibrationHistoryViewMapper mapper = context.getBean(SpaceCalibrationHistoryViewMapper.class);
        SpaceCalibrationHistoryView record = new SpaceCalibrationHistoryView();
        XCopyUtils.copyObject(input, record);
        SpaceCalibrationHistoryView view = mapper.selectOne(record);
        if (view == null) {
            return XSingleResult.error(context, XError.NotFound);
        }
        GetSpaceCalibrationHistoryViewOutput output = new GetSpaceCalibrationHistoryViewOutput();
        XCopyUtils.copyObject(view, output);
        return XSingleResult.success(output);
    }

    @XApiAnonymous
    @XApiGet
    public XPageResult<QuerySpaceCalibrationHistoryViewOutput> querySpaceCalibrationHistoryView(XContext context, QuerySpaceCalibrationHistoryViewInput input) {
        SpaceCalibrationHistoryViewMapper mapper = context.getBean(SpaceCalibrationHistoryViewMapper.class);
        SpaceCalibrationHistoryView record = new SpaceCalibrationHistoryView();
        XCopyUtils.copyObject(input, record);
        PageHelper.startPage(input.getPage(), input.getLimit());
        List<SpaceCalibrationHistoryView> list = mapper.selectList(record);
        PageInfo<SpaceCalibrationHistoryView> pageInfo = new PageInfo<>(list);
        List<QuerySpaceCalibrationHistoryViewOutput> outputs = XCopyUtils.copyNewList(pageInfo.getList(), QuerySpaceCalibrationHistoryViewOutput.class);
        return XPageResult.success(outputs, input, pageInfo.getTotal());
    }

}
